@@ -4,6 +4,7 @@
 """
 from __future__ import absolute_import
 
+import sys
 import logging
 import fileinput
 from email.parser import FeedParser
@@ -19,8 +20,10 @@ manager = Manager(description='SBE forum commands',
                   help='SBE forum commands')
 
 
-@manager.command
-def inject_email():
+@manager.option('-f', '--filename',
+                help='email filename; defaults to standard input',
+                default=u'-', required=False,)
+def inject_email(filename=u'-'):
   """
   Reads one email from stdin,
   parse it,
@@ -28,12 +31,20 @@ def inject_email():
   """
   parser = FeedParser()
   message = None
+
+  if logger.level is logging.NOTSET:
+    logger.setLevel(logging.INFO)
+
   try:
     # iterate over stdin
-    for line in fileinput.input('-'):
+    for line in fileinput.input(filename):
       parser.feed(line)
+  except KeyboardInterrupt:
+    logger.info('Aborted by user, exiting.')
+    sys.exit(1)
   except:
     logger.error('Error during email parsing', exc_info=True)
+    sys.exit(1)
   finally:
     # close the parser to generate a email.message
     message = parser.close()
@@ -44,9 +55,14 @@ def inject_email():
     if len(message.defects) == 0:
       process_email.delay(message)
     else:
-      logger.error('email has defects')
+      logger.error('email has defects, message content:\n'
+                   '------ START -------\n'
+                   '%s'
+                   '\n------ END -------\n',
+                   message,
+                   extra={ 'stack': True, })
   else:
-    logger.error('no email was parsed from stdin')
+    logger.error('no email was parsed from stdin', extra={ 'stack': True, })
 
 
 @manager.command
