@@ -18,6 +18,11 @@ def register_plugin(app):
 
   # TODO: Slightly confusing. Reorg?
   from .views import notifications, social  # noqa
-  from .tasks import social  # noqa
+  from .tasks.social import DIGEST_TASK_NAME, DEFAULT_DIGEST_SCHEDULE
+
+  CELERYBEAT_SCHEDULE = app.config.setdefault('CELERYBEAT_SCHEDULE', {})
+
+  if DIGEST_TASK_NAME not in CELERYBEAT_SCHEDULE:
+    CELERYBEAT_SCHEDULE[DIGEST_TASK_NAME] = DEFAULT_DIGEST_SCHEDULE
 
   app.register_blueprint(notifications)
